@@ -245,6 +245,33 @@ class VCard {
 		}
 	}
 
+	/**
+	* Work around issue in older VObject sersions
+	* https://github.com/fruux/sabre-vobject/issues/24
+	* @param $vcard Reference to a Sabre_VObject_Property.
+	*/
+	public static function fixPropertyParameters(&$vcard) {
+		// Work around issue in older VObject sersions
+		// https://github.com/fruux/sabre-vobject/issues/24
+		foreach($vcard->children as $property) {
+			foreach($property->parameters as $key=>$parameter) {
+				$delim = '';
+				if(strpos($parameter->value, ',') !== false) {
+					$delim = ',';
+				} elseif(strpos($parameter->value, '\\,') !== false) {
+					$delim = '\\,';
+				} else {
+					continue;
+				}
+				$values = explode($delim, $parameter->value);
+				$parameter->value = array_shift($values);
+				foreach($values as $value) {
+					$property->add($parameter->name, $value);
+				}
+			}
+		}
+	}
+
 	/**
 	* @brief Checks if a contact with the same UID already exist in the address book.
 	* @param $aid Address book ID.
@@ -301,6 +328,7 @@ class VCard {
 			// Decode string properties and remove obsolete properties.
 			if($upgrade) {
 				self::decodeProperty($property);
+				self::fixPropertyParameters($vcard);
 			}
 			if(function_exists('iconv')) {
 				$property->value = str_replace("\r\n", "\n", iconv(mb_detect_encoding($property->value, 'UTF-8, ISO-8859-1'), 'utf-8', $property->value));
@@ -581,6 +609,9 @@ class VCard {
 				', Unable to parse VCARD, : ' . $e->getMessage(), \OCP\Util::ERROR);
 			return false;
 		}
+
+		self::fixPropertyParameters($vcard);
+
 		try {
 			self::edit($oldcard['id'], $vcard);
 			return true;
@@ -859,10 +890,11 @@ class VCard {
 		if ($addressbook['userid'] != \OCP\User::getUser()) {
 			$sharedAddressbook = \OCP\Share::getItemSharedWithBySource('addressbook', $aid);
 			if (!$sharedAddressbook || !($sharedAddressbook['permissions'] & \OCP\PERMISSION_CREATE)) {
-				return false;
+				throw new \Exception(App::$l10n->t('You don\'t have permissions to move contacts into this address book'));
 			}
 		}
 		if(is_array($id)) {
+			// NOTE: This block is currently not used and need rewrite if used!
 			foreach ($id as $index => $cardId) {
 				$card = self::find($cardId);
 				if (!$card) {
@@ -885,13 +917,13 @@ class VCard {
 				$result = $stmt->execute($vals);
 				if (\OC_DB::isError($result)) {
 					\OC_Log::write('contacts', __METHOD__. 'DB error: ' . \OC_DB::getErrorMessage($result), \OC_Log::ERROR);
-					return false;
+					throw new \Exception(App::$l10n->t('Database error during move.'));
 				}
 			} catch(\Exception $e) {
 				\OCP\Util::writeLog('contacts', __METHOD__.', exception: '.$e->getMessage(), \OCP\Util::ERROR);
 				\OCP\Util::writeLog('contacts', __METHOD__.', ids: '.join(',', $vals), \OCP\Util::DEBUG);
 				\OCP\Util::writeLog('contacts', __METHOD__.', SQL:'.$prep, \OCP\Util::DEBUG);
-				return false;
+				throw new \Exception(App::$l10n->t('Database error during move.'));
 			}
 		} else {
 			$stmt = null;
@@ -900,27 +932,28 @@ class VCard {
 			} else {
 				$card = self::find($id);
 				if (!$card) {
-					return false;
+					throw new \Exception(App::$l10n->t('Error finding card to move.'));
 				}
 				$oldAddressbook = Addressbook::find($card['addressbookid']);
 				if ($oldAddressbook['userid'] != \OCP\User::getUser()) {
-					$sharedContact = \OCP\Share::getItemSharedWithBySource('contact', $id, \OCP\Share::FORMAT_NONE, null, true);
-					if (!$sharedContact || !($sharedContact['permissions'] & \OCP\PERMISSION_DELETE)) {
-						return false;
+					$sharedAddressbook = \OCP\Share::getItemSharedWithBySource('addressbook', $oldAddressbook['id']);
+					if (!$sharedAddressbook || !($sharedAddressbook['permissions'] & \OCP\PERMISSION_DELETE)) {
+						throw new \Exception(App::$l10n->t('You don\'t have permissions to move contacts from this address book'));
 					}
 				}
+				Addressbook::touch($oldAddressbook['id']);
 				$stmt = \OCP\DB::prepare( 'UPDATE `*PREFIX*contacts_cards` SET `addressbookid` = ? WHERE `id` = ?' );
 			}
 			try {
 				$result = $stmt->execute(array($aid, $id));
 				if (\OC_DB::isError($result)) {
 					\OC_Log::write('contacts', __METHOD__. 'DB error: ' . \OC_DB::getErrorMessage($result), \OC_Log::ERROR);
-					return false;
+					throw new \Exception(App::$l10n->t('Database error during move.'));
 				}
 			} catch(\Exception $e) {
 				\OCP\Util::writeLog('contacts', __METHOD__.', exception: '.$e->getMessage(), \OCP\Util::DEBUG);
 				\OCP\Util::writeLog('contacts', __METHOD__.' id: '.$id, \OCP\Util::DEBUG);
-				return false;
+				throw new \Exception(App::$l10n->t('Database error during move.'));
 			}
 		}
 		\OC_Hook::emit('\OCA\Contacts\VCard', 'post_moveToAddressbook', array('aid' => $aid, 'id' => $id));
