@@ -14,15 +14,26 @@ $tmpl = new OCP\Template('files_encryption', 'settings-personal');
 $user = \OCP\USER::getUser();
 $view = new \OC_FilesystemView('/');
 $util = new \OCA\Encryption\Util($view, $user);
+$session = new \OCA\Encryption\Session($view);
+
+$privateKeySet = ($session->getPrivateKey() !== false) ? true : false;
 
 $recoveryAdminEnabled = OC_Appconfig::getValue('files_encryption', 'recoveryAdminEnabled');
 $recoveryEnabledForUser = $util->recoveryEnabledForUser();
 
-\OCP\Util::addscript('files_encryption', 'settings-personal');
-\OCP\Util::addScript('settings', 'personal');
+$result = false;
 
-$tmpl->assign('recoveryEnabled', $recoveryAdminEnabled);
-$tmpl->assign('recoveryEnabledForUser', $recoveryEnabledForUser);
+if ($recoveryAdminEnabled || !$privateKeySet) {
 
-return $tmpl->fetchPage();
+	\OCP\Util::addscript('files_encryption', 'settings-personal');
+	\OCP\Util::addScript('settings', 'personal');
+
+	$tmpl->assign('recoveryEnabled', $recoveryAdminEnabled);
+	$tmpl->assign('recoveryEnabledForUser', $recoveryEnabledForUser);
+	$tmpl->assign('privateKeySet', $privateKeySet);
+
+	$result = $tmpl->fetchPage();
+}
+
+return $result;
 
